// Direct.h
//
// Declarations for Direct3D support classes
//

class CMatrix : public _D3DMATRIX
{
public:
	CMatrix(IDirect3DDevice* pID3D = NULL);
	virtual ~CMatrix();
	Create(IDirect3DDevice* pID3D);
	operator D3DMATRIXHANDLE() const;
	void Update();

protected:
	D3DMATRIXHANDLE m_hMatrix;
	IDirect3DDevice* m_pID3D;

};

class CExecute 
{
public:
	CExecute(IDirect3DDevice* pID3D, IDirect3DViewport* pIViewport);
	virtual ~CExecute();
	operator IDirect3DExecuteBuffer*() const {return m_pIEB;}
	BOOL AddByte(BYTE b);
	BOOL AddStruct(void* pStruct, int iSize);
	BOOL AddIns(D3DOPCODE op, BYTE size, WORD count);
	BOOL AddState(D3DTRANSFORMSTATETYPE t, D3DMATRIXHANDLE h);
	BOOL Execute();
	BOOL AddAmbLight(D3DCOLOR clr);
	BOOL AddVertices(D3DVERTEX* pVert, int nVert);
	BOOL AddVertices(D3DTLVERTEX* pVert, int nVert);
	BOOL AddProcess(int nVert,
					int iStart = 0,
					DWORD dwFlags = D3DPROCESSVERTICES_TRANSFORMLIGHT);
	BOOL AddLine(int v1, int v2);
	BOOL AddLines(D3DLINE* pLines, int nLines);
	BOOL AddPoint(int v);
	BOOL AddPoints(D3DPOINT* pPointInfo);
	BOOL AddTriangle(int v1, int v2, int v3);
	BOOL AddMaterial(D3DMATERIALHANDLE hMat);
	BOOL GetExecuteData(D3DEXECUTEDATA* ped);
	void* LockBuffer();
	BOOL AddRenderState(D3DRENDERSTATETYPE state, DWORD value);
	BOOL AddMatMul(D3DMATRIXHANDLE hDest,
				   D3DMATRIXHANDLE hSrc1,
				   D3DMATRIXHANDLE hSrc2);

protected:
	IDirect3DDevice* m_pID3D;
	IDirect3DViewport* m_pIViewport;
	IDirect3DExecuteBuffer* m_pIEB;
	D3DEXECUTEBUFFERDESC m_Desc;
	DWORD m_dwOffset;		// offset to next free entry
	DWORD m_dwInsOffset;	// offset to first instruction
};

class CMaterial
{
public:
	CMaterial();
	virtual ~CMaterial();
	BOOL Create(IDirect3D* pIEngine, IDirect3DDevice* pIDevice);
	operator D3DMATERIALHANDLE() const;
	operator IDirect3DMaterial*() const;
	BOOL SetColor(double r, double g, double b);
	BOOL SetShades(int n);

protected:
	D3DMATERIAL m_Mat;
	D3DMATERIALHANDLE m_hMat;
	IDirect3DMaterial* m_pIMat;
};

class CLight
{
public:
	CLight(IDirect3D* pIEngine = NULL);
	virtual ~CLight();
	BOOL Create(IDirect3D* pIEngine);
	operator IDirect3DLight*() const;
	void SetType(D3DLIGHTTYPE type);
	void SetColor(double r, double g, double b);
	void SetDirection(double x, double y, double z);
	void SetPosition(double x, double y, double z);
	BOOL Update();

protected:
	D3DLIGHT m_Light;
	IDirect3DLight* m_pILight;
};
