// TestWnd.cpp : implementation file
//

#include "stdafx.h"
#include "D3DEval.h"
#include "Direct.h"
#include "TestWnd.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define D2R  0.01745329251994

/////////////////////////////////////////////////////////////////////////////
// CTestWnd

CTestWnd::CTestWnd()
{
	m_pTI = NULL;
	m_pDD = NULL;
	m_pD3D = NULL;
	m_pIEngine = NULL;
	m_pIDevice = NULL;
	m_pIViewport = NULL;
}

CTestWnd::~CTestWnd()
{
	delete m_pD3D;
	delete m_pDD;
	delete m_pTI;
	// don't release m_pIEngine or m_pIDevice as
	// they weren't addref'd
	if (m_pIViewport) {
		// BUGBUG: Don't release the viewport - it's already dead
		//m_pIViewport->Release();
	}
}


BEGIN_MESSAGE_MAP(CTestWnd, CWnd)
	//{{AFX_MSG_MAP(CTestWnd)
	ON_WM_CHAR()
	ON_WM_DESTROY()
	ON_WM_ERASEBKGND()
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////

// Create the test window
BOOL CTestWnd::Create(TESTINFO* pti)
{
	// Save the pointer to the test info
	m_pTI = pti;
	ASSERT(m_pTI);

	// Create a DirectDraw object
	m_pDD = new CDirectDraw;
	BOOL b = m_pDD->Create();
	ASSERT(b);

	// Register window class for test window
	CString strClass =
		AfxRegisterWndClass(CS_HREDRAW | CS_VREDRAW,
							::LoadCursor(NULL, IDC_ARROW),
							(HBRUSH)::GetStockObject(GRAY_BRUSH));

	// Define window style and size
	DWORD dwStyle = WS_VISIBLE | WS_POPUP;
	RECT rc;
	if (m_pTI->bFullScreen) {
		rc.top = 0;
		rc.left = 0;
		rc.right = ::GetSystemMetrics(SM_CXSCREEN);
		rc.bottom = ::GetSystemMetrics(SM_CYSCREEN);
	} else { // Windowed
		dwStyle |= WS_CAPTION | WS_SYSMENU;
		rc.top = 50;
		rc.left = 50;
		rc.bottom = rc.top + m_pTI->iHeight;
		rc.right = rc.left + m_pTI->iWidth;

		// Adjust the window size so its client area is
		// requested size
		::AdjustWindowRect(&rc, dwStyle, FALSE);
	}

	// Create the window. 
	// We have no WM_CREATE handler, so not much
	// happens here.
	if (!CreateEx(0,
				  strClass,
				  "Direct3D Window",
				  dwStyle,
				  rc.left, rc.top,
				  rc.right - rc.left, rc.bottom - rc.top,
				  m_pTI->pParent->GetSafeHwnd(),
				  NULL)) {
		return FALSE;
	}

	// Ensure window is visible
	UpdateWindow();

	// Set mode for window in DirectDraw object,
	// which creates front and back buffers and palette
	// if one is required
	if (m_pTI->bFullScreen) {
		b = m_pDD->SetFullScreenMode(GetSafeHwnd(),
									 m_pTI->iWidth,
									 m_pTI->iHeight,
									 m_pTI->iBpp);
	} else {
		b = m_pDD->SetWindowedMode(GetSafeHwnd(),
								   m_pTI->iWidth,
								   m_pTI->iHeight);
	}
	ASSERT(b);

	// Create Direct3D object on top of DirectDraw surfaces
	m_pD3D = new CDirect3D();
	b = m_pD3D->Create(m_pDD);
	ASSERT(b);

	// Set lighting model, which selects either a hardware or
	// software driver and also creates Z buffer
	if (pti->iLightMode == 1) {
		b = m_pD3D->SetMode(D3DCOLOR_MONO);
	} else {
		b = m_pD3D->SetMode(D3DCOLOR_RGB);
	}
	ASSERT(b);

	// Get the D3D engine and device interface pointers 
	m_pIEngine = m_pD3D->GetD3DEngine();
	ASSERT(m_pIEngine);
	m_pIDevice = m_pD3D->GetD3DDevice();
	ASSERT(m_pIDevice);

	// Create viewport
	HRESULT hr = m_pIEngine->CreateViewport(&m_pIViewport,
											NULL);
	ASSERT(SUCCEEDED(hr));
	ASSERT(m_pIViewport);

	// Attach viewport to device
	hr = m_pIDevice->AddViewport(m_pIViewport);
	ASSERT(SUCCEEDED(hr));

	// Configure viewport.
	// Note: Some details here relate to scale factors and
	// so on, which will get set again later when we set up
	// projecttion matrix.
	D3DVIEWPORT vd;
	memset(&vd, 0, sizeof(vd));
	vd.dwSize = sizeof(vd);		// Structure size

	// Define viewport area on device
    vd.dwX = 0;	   // Left side
    vd.dwY = 0;    // Top
    vd.dwWidth = m_pTI->iWidth;   // Width 
    vd.dwHeight = m_pTI->iHeight; // Height

	// Set scale so that viewport is 2-by-2 model units
    vd.dvScaleX = D3DVAL(m_pTI->iWidth) / D3DVAL(2.0);
    vd.dvScaleY = D3DVAL(m_pTI->iHeight) / D3DVAL(2.0);
	
	// Set maximum x and y values to 1 so origin is
	// in the center and the x and y ranges vary from -1 to +1,
	// i.e., -range/2 to +range/2
    vd.dvMaxX = D3DVAL(1.0);
    vd.dvMaxY = D3DVAL(1.0);

	// Set Z range
    vd.dvMinZ = D3DVAL(-5.0);
    vd.dvMaxZ = D3DVAL(100.0);

	// Apply setings to viewport
	hr = m_pIViewport->SetViewport(&vd);
	ASSERT(SUCCEEDED(hr));

	// Set rendering characteristics.
	// Create an execute buffer.
	CExecute ex1 (m_pIDevice, m_pIViewport);

	// Set a shade mode and fill mode. Direct3D selects the
	// drivers it will use based on these values, and it
	// has no default.
	ex1.AddRenderState(D3DRENDERSTATE_FILLMODE, D3DFILL_SOLID);
	ex1.AddRenderState(D3DRENDERSTATE_SHADEMODE, D3DSHADE_GOURAUD);
	ex1.AddRenderState(D3DRENDERSTATE_DITHERENABLE, 1);

	// Enable Z buffer
	ex1.AddRenderState(D3DRENDERSTATE_ZENABLE, 1);

	// Now set up the initial scene state

	// Create matrix for the world transform (identity)
	m_mWorld.Create(m_pIDevice);
	ex1.AddState(D3DTRANSFORMSTATE_WORLD, m_mWorld);

	// Create matrix for projection.
	// Note: We did this when we set up the viewport
	// earlier, but when we selected the fill mode and
	// shading mode the viewport data was reset.
	m_mProjection.Create(m_pIDevice);
	m_mProjection._11 = D3DVAL(2);
	m_mProjection._22 = D3DVAL(2);
	m_mProjection._34 = D3DVAL(1);
	m_mProjection._43 = D3DVAL(-1);
	m_mProjection._44 = D3DVAL(0);
	m_mProjection.Update();
	ex1.AddState(D3DTRANSFORMSTATE_PROJECTION,
				 m_mProjection);

	// Create matrix for view
	// (the effective camera position)
	m_mView.Create(m_pIDevice);
	m_mView._43 = D3DVAL(2); // Camera Z = -2;
	m_mView.Update();
	ex1.AddState(D3DTRANSFORMSTATE_VIEW, m_mView);

	// Add command to set ambient light level
	ex1.AddAmbLight(RGB_MAKE(64, 64, 64));

	// Execute command list
	b = ex1.Execute();
	ASSERT(b);

	// Add light source to viewport
	m_Light1.Create(m_pIEngine);
	m_Light1.SetColor(0.8, 0.8, 0.8);
	m_Light1.SetType(D3DLIGHT_DIRECTIONAL);
	m_Light1.SetDirection(1, -1, 1);
	m_Light1.Update();
	hr = m_pIViewport->AddLight(m_Light1);
	ASSERT(SUCCEEDED(hr));

	// Create material for background.
	// Note: We must set the number of shades to 1, or the
	// Mono driver sets the color to black.
	// Of course, we need only one shade for the background
	// color, so this avoids wasted palette space.
	m_matBkgnd.Create(m_pIEngine, m_pIDevice);
	m_matBkgnd.SetColor(0.0, 0.0, 0.5);	// Dark blue
	m_matBkgnd.SetShades(1); // Only one shade
	hr = m_pIViewport->SetBackground(m_matBkgnd);
	ASSERT(SUCCEEDED(hr));

	
	// Run the tests
	SetTimer(1, 100, NULL);
	return TRUE;
}

void CTestWnd::OnTimer(UINT nIDEvent) 
{
	// Run the next test
	switch (m_pTI->iTest) {
	case 1:
		Test1();
		break;
	case 2:
		if (m_pTI->iLightMode == 1) {
			Test2MONO();
		} else {
			Test2RGB();
		}
		break;
	default:
		ASSERT(0);
		break;
	}
}

void CTestWnd::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// Any key ends the test
	DestroyWindow();
}

void CTestWnd::OnDestroy() 
{
	// Stop the update timer
	KillTimer(1);

	// Restore the screen mode
	ASSERT(m_pTI);
	if (m_pTI->bFullScreen) {
		m_pDD->RestoreMode();
	}

	CWnd::OnDestroy();
}

void CTestWnd::PostNcDestroy() 
{
	// delete this C++ window container
	// Gotta love those MFC guys eh?
	delete this;
}

// Disable the default background erase
BOOL CTestWnd::OnEraseBkgnd(CDC* pDC) 
{
	return TRUE;
}


void CTestWnd::Test1()
{
	HRESULT hr;

	ASSERT(m_pTI);

	// Get pointers to front and back buffers
	CDDSurface* pBB = m_pDD->GetBackBuffer();
	ASSERT(pBB);
	CDDSurface* pFB = m_pDD->GetFrontBuffer();
	ASSERT(pFB);

	// Get rectangle that describes front buffer
	RECT rcFront;
	if (m_pTI->bFullScreen) {
		pFB->GetRect(rcFront);
	} else {
		GetClientRect(&rcFront);
		ClientToScreen(&rcFront);
	}

	// Get rectangle that describes back buffer
	RECT rcBack;
	pBB->GetRect(rcBack);

	// Create vertex list for shape (a pyramid of sorts)
	D3DVERTEX vShape [] = {
		{	D3DVAL(-0.3), D3DVAL(-0.1), D3DVAL( 0.1), // Vertex xyz
			D3DVAL(-1.0), D3DVAL(-1.0), D3DVAL(-1.0), // Normal xyz
			D3DVAL( 0.0), D3DVAL( 0.0)				  // Texture uv
		},
		{	D3DVAL( 0.3), D3DVAL(-0.1), D3DVAL( 0.2),
			D3DVAL( 1.0), D3DVAL(-1.0), D3DVAL(-1.0),
			D3DVAL( 0.0), D3DVAL( 0.0)
		},
		{	D3DVAL( 0.0), D3DVAL(-0.3), D3DVAL( 0.3),
			D3DVAL( 0.0), D3DVAL(-1.0), D3DVAL( 1.0),
			D3DVAL( 0.0), D3DVAL( 0.0)
		},
		{	D3DVAL( 0.1), D3DVAL( 0.4), D3DVAL( 0.3),
			D3DVAL( 0.0), D3DVAL( 1.0), D3DVAL( 0.0),
			D3DVAL( 0.0), D3DVAL( 0.0)
		}
	};
	int nVerts = sizeof(vShape) / sizeof(D3DVERTEX);

	// Create material for shape
	CMaterial mShape;
	mShape.Create(m_pIEngine, m_pIDevice);
	mShape.SetColor(0.0, 1.0, 0.0);	// Bright green

	// Create execute buffer
	CExecute ex (m_pIDevice, m_pIViewport);

	// Add vertex data to buffer
	ex.AddVertices(vShape, nVerts);

	// Create matrix for world rotation
	CMatrix mRot(m_pIDevice);
	double ry = 1; // Degrees about y axis
	double siny = sin(ry * D2R);
	double cosy = cos(ry * D2R);
	mRot._11 = D3DVAL(cosy);
	mRot._13 = D3DVAL(-siny);
	mRot._31 = D3DVAL(siny);
	mRot._33 = D3DVAL(cosy);
	mRot.Update();
	
	// Add command to multiply world matrix by rotation
	// matrix
	ex.AddMatMul(m_mWorld, m_mWorld, mRot);

	// Add shape material to execute buffer
	ex.AddMaterial(mShape);

	// Add processing command for vertices so
	// each is transformed and lit
	ex.AddProcess(nVerts, 0,
				  D3DPROCESSVERTICES_TRANSFORMLIGHT|
				  D3DPROCESSVERTICES_UPDATEEXTENTS);

	// Add a set of commands to draw shape
#if 0 // Set to 1 to draw lines
	ex.AddLine(0, 1);
	ex.AddLine(1, 2);
	ex.AddLine(2, 0);
	ex.AddLine(0, 3);
	ex.AddLine(1, 3);
	ex.AddLine(2, 3);
#else // Use triangles
	ex.AddTriangle(0, 3, 1);
	ex.AddTriangle(1, 3, 2);
	ex.AddTriangle(2, 3, 0);
	ex.AddTriangle(0, 1, 2);
#endif

	DWORD dwStart = timeGetTime();
	int nFrames = 360;
	for (int iFrame = 0; iFrame < nFrames; iFrame++) {

		DWORD dwNow = timeGetTime();
		double fps; 
		if (dwNow == dwStart) {
			fps = 0;
		} else {
			fps = iFrame * 1000.0 / (double)(dwNow - dwStart);
		}

		// Clear viewport (to current background material)
		D3DRECT r;
		r.x1 = 0;
		r.y1 = 0; // Top-left
		r.x2 = m_pTI->iWidth;
		r.y2 = m_pTI->iHeight; // Bottom-right
		hr = m_pIViewport->Clear(1, &r, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER);
		ASSERT(SUCCEEDED(hr));

#if 1 // Set to zero to remove frame rate text.
		// Draw frame rate text to back buffer.
		char buf[64];
		sprintf(buf, "Frame %2d  (%3.1f fps)", iFrame, fps);
		CDC* pdc = pBB->GetDC();
		ASSERT(pdc);
		pdc->DrawText(buf, -1, &rcBack, DT_CENTER | DT_BOTTOM | DT_SINGLELINE);
		pBB->ReleaseDC(pdc);
#endif

		// Execute buffer
		BOOL b = ex.Execute();
		ASSERT(b);

		// Swap buffers
		if (m_pTI->bFullScreen) {
			pFB->Flip();
		} else {
			pFB->Blt(&rcFront, pBB, &rcBack);
		}
	}
}

void CTestWnd::Test2RGB()
{
	ASSERT(m_pTI);

	// Get the front and back buffers
	CDDSurface* pBB = m_pDD->GetBackBuffer();
	ASSERT(pBB);
	CDDSurface* pFB = m_pDD->GetFrontBuffer();
	ASSERT(pFB);

	// Get the rectangle that describes the front buffer
	RECT rcFront;
	if (m_pTI->bFullScreen) {
		pFB->GetRect(rcFront);
	} else {
		GetClientRect(&rcFront);
		ClientToScreen(&rcFront);
	}

	RECT rcBack;
	pBB->GetRect(rcBack);

	// Create a vertex list for a shape
	// Using transformed and lit coords
	D3DTLVERTEX vShape [] = {
		{	D3DVAL(10), D3DVAL(10), D3DVAL(2), D3DVAL(1), // x, y, z, 1/w
			RGBA_MAKE(255, 0, 0, 255), RGBA_MAKE(255, 255, 255, 255), // Color, specular color
			D3DVAL(0), D3DVAL(0)// Texture u, v
		},
		{	D3DVAL(m_pTI->iWidth - 10), D3DVAL( 10), D3DVAL(2), D3DVAL(1),
			RGBA_MAKE(0, 0, 255, 255), RGBA_MAKE(255, 255, 255, 255),
			D3DVAL(0), D3DVAL(0)
		},
		{	D3DVAL( 10), D3DVAL(m_pTI->iHeight - 10), D3DVAL(2), D3DVAL(1),
			RGBA_MAKE(0, 255, 0, 255), RGBA_MAKE(255, 255, 255, 255),
			D3DVAL(0), D3DVAL(0)
		}
	};
	int nVerts = sizeof(vShape) / sizeof(D3DTLVERTEX);

	// Create execute buffer
	CExecute ex (m_pIDevice, m_pIViewport);

	// Add vertex data to buffer
	ex.AddVertices(vShape, nVerts);

	// Add processing command for vertices so
	// each gets transformed and lit
	ex.AddProcess(nVerts, 0, D3DPROCESSVERTICES_COPY|
				  D3DPROCESSVERTICES_UPDATEEXTENTS);

	// Add commands to draw shape
	ex.AddTriangle(0, 1, 2);

	DWORD dwStart = timeGetTime();
	int nFrames = 100;
	for (int iFrame = 0; iFrame < nFrames; iFrame++) {

		DWORD dwNow = timeGetTime();
		double fps; 
		if (dwNow == dwStart) {
			fps = 0;
		} else {
			fps = iFrame * 1000.0 / (double)(dwNow - dwStart);
		}

		// Clear the viewport (to the current background material)
		D3DRECT r;
		r.x1 = 0;
		r.y1 = 0; // top-left
		r.x2 = m_pTI->iWidth - 1;
		r.y2 = m_pTI->iHeight - 1; // bottom-right
		HRESULT hr = m_pIViewport->Clear(1, &r, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER);
		ASSERT(SUCCEEDED(hr));

#if 1
		// Draw the frame rate text to the back buffer
		char buf[64];
		sprintf(buf, "Frame %2d  (%3.1f fps)", iFrame, fps);
		CDC* pdc = pBB->GetDC();
		ASSERT(pdc);
		pdc->DrawText(buf, -1, &rcBack, DT_CENTER | DT_BOTTOM | DT_SINGLELINE);
		pBB->ReleaseDC(pdc);
#endif

		// Execute the buffer
		BOOL b = ex.Execute();
		ASSERT(b);

		// Swap buffers
		if (m_pTI->bFullScreen) {
			pFB->Flip();
		} else {
			pFB->Blt(&rcFront, pBB, &rcBack);
		}
	}
}

void CTestWnd::Test2MONO()
{
	ASSERT(m_pTI);

	// Get the front and back buffers
	CDDSurface* pBB = m_pDD->GetBackBuffer();
	ASSERT(pBB);
	CDDSurface* pFB = m_pDD->GetFrontBuffer();
	ASSERT(pFB);

	// Get the rectangle that describes the front buffer
	RECT rcFront;
	if (m_pTI->bFullScreen) {
		pFB->GetRect(rcFront);
	} else {
		GetClientRect(&rcFront);
		ClientToScreen(&rcFront);
	}

	RECT rcBack;
	pBB->GetRect(rcBack);

	// We are using the MONO rasterizer so we need a material to define the color
	CMaterial mShape;
	mShape.Create(m_pIEngine, m_pIDevice);
	mShape.SetColor(0.0, 1.0, 0.0);	// Bright green
		
	// Create a vertex list for a shape using transformed and lit coords
	// The blue channel of 'color' values are shades (fractions) of the material color
	D3DTLVERTEX vShape [] = {
		{	D3DVAL(10), D3DVAL(10), D3DVAL(2), D3DVAL(1), // x, y, z, 1/w
			RGBA_MAKE(0, 0, 64, 0), RGBA_MAKE(0, 0, 64, 0), // Color, specular color
			D3DVAL(0), D3DVAL(0)// Texture u, v
		},
		{	D3DVAL(m_pTI->iWidth - 10), D3DVAL( 10), D3DVAL(2), D3DVAL(1),
			RGBA_MAKE(0, 0, 255, 0), RGBA_MAKE(0, 0, 255, 0),
			D3DVAL(0), D3DVAL(0)
		},
		{	D3DVAL( 10), D3DVAL(m_pTI->iHeight - 10), D3DVAL(2), D3DVAL(1),
			RGBA_MAKE(0, 0, 0, 0), RGBA_MAKE(0, 0, 0, 0),
			D3DVAL(0), D3DVAL(0)
		}
	};
	int nVerts = sizeof(vShape) / sizeof(D3DTLVERTEX);

	// Create execute buffer
	CExecute ex (m_pIDevice, m_pIViewport);

	// Add vertex data to buffer
	ex.AddVertices(vShape, nVerts);

	// Add shape material to execute buffer
	ex.AddMaterial(mShape);

	// Add processing command for vertices so
	// each gets transformed and lit
	ex.AddProcess(nVerts, 0, D3DPROCESSVERTICES_COPY|
				  D3DPROCESSVERTICES_UPDATEEXTENTS);

	// Add a set of commands to draw the shape
	ex.AddTriangle(0, 1, 2);

	DWORD dwStart = timeGetTime();
	int nFrames = 100;
	for (int iFrame = 0; iFrame < nFrames; iFrame++) {

		DWORD dwNow = timeGetTime();
		double fps; 
		if (dwNow == dwStart) {
			fps = 0;
		} else {
			fps = iFrame * 1000.0 / (double)(dwNow - dwStart);
		}

		// Clear the viewport (to the current background material)
		D3DRECT r;
		r.x1 = 0;
		r.y1 = 0; // Top-left
		r.x2 = m_pTI->iWidth - 1;
		r.y2 = m_pTI->iHeight - 1; // Bottom-right
		HRESULT hr = m_pIViewport->Clear(1, &r, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER);
		ASSERT(SUCCEEDED(hr));

#if 1
		// Draw the frame rate text to the back buffer
		char buf[64];
		sprintf(buf, "Frame %2d  (%3.1f fps)", iFrame, fps);
		CDC* pdc = pBB->GetDC();
		ASSERT(pdc);
		pdc->DrawText(buf, -1, &rcBack, DT_CENTER | DT_BOTTOM | DT_SINGLELINE);
		pBB->ReleaseDC(pdc);
#endif

		// Execute the buffer
		BOOL b = ex.Execute();
		ASSERT(b);

		// Swap buffers
		if (m_pTI->bFullScreen) {
			pFB->Flip();
		} else {
			pFB->Blt(&rcFront, pBB, &rcBack);
		}
	}
}

